require "narray"
require "numru/dcl"
#require "advanceddcl/parameter_appli"
require "numru/advanceddcl/parameter_appli"
require "numru/advanceddcl/parameter"
#include NumRu
require "numru/advanceddcl/common"
require "numru/advanceddcl/dev"
require "numru/advanceddcl/frame"
require "numru/advanceddcl/fig"
require "numru/advanceddcl/page"
require "numru/advanceddcl/basic"
require "numru/advanceddcl/contour"
require "numru/advanceddcl/tone"
require "numru/advanceddcl/vector"
require "numru/advanceddcl/mark"
require "numru/advanceddcl/plot"
require "numru/advanceddcl/text"
require "numru/advanceddcl/axis"
require "numru/advanceddcl/dateaxis"
require "numru/advanceddcl/arrow"
require "numru/advanceddcl/errbar"
require "numru/advanceddcl/gap"
require "numru/advanceddcl/bar"
require "numru/advanceddcl/box"
require "numru/advanceddcl/map"
require "numru/advanceddcl/three-d"

# by kawanabe
#english
=begin
=AdvancedDCL
This library is wrapper of RubyDCL and it is more useful with Ruby than RubyDCL. One of differences between RubyDCL and AdvancedDCL is a way of handling parameters. In this library, it is possible to set parameters as a keyword argument of Ruby methods. This setting is temporary setting. If you need more explanation, refer ((<About setting parameters>)), please.

DCL online manual page

((<URL:http://www.gfd-dennou.org/arch/dcl/dcl-f77doc/rc1/ >))

Modules in AdvancedDCL are follows. About methods in each module, please refer ((<Modules of this library>)).
  * module name      explanation    (corresponding DCL packages)
* NumRu
  * AdvancedDCL
    * ((<Common|NumRu::AdvancedDCL::Common>)) handring common parameters (None)
    * ((<Dev|NumRu::AdvancedDCL::Dev>)) module for device (SWPACK)
    * ((<Page|NumRu::AdvancedDCL::Page>)) module for pages (SLPACK)
    * ((<Frame|NumRu::AdvancedDCL::Frame>)) module for frame (SLPACK)
    * ((<Fig|NumRu::AdvancedDCL::Fig>)) module for design of figure (window, viewport)(SGPACK, GRPACK, UMPACK, SCPACK)
    * ((<Axis|NumRu::AdvancedDCL::Axis>)) module for auto drawing of axis (USPACK)
      * ((<Lin|NumRu::AdvancedDCL::Axis::Lin>)) module for drawing linear axis (U[XYZ]PACK)
      * ((<Log|NumRu::AdvancedDCL::Axis::Log>)) module for log axis (ULPACK)
    * ((<Dateaxis|NumRu::AdvancedDCL::Dateaxis>)) module for date axis (UCPACK)
    * ((<Plot|NumRu::AdvancedDCL::Plot>)) module for plotting (UUPACK)
    * ((<Contour|NumRu::AdvancedDCL::Contour>)) module for drawing contour (UDPACK)
    * ((<Tone|NumRu::AdvancedDCL::Tone>)) module for drawing mono and color tone (UEPACK)
    * ((<Vector|NumRu::AdvancedDCL::Vector>)) module for drawing vector (UGPACK)
    * ((<Map|NumRu::AdvancedDCL::Map>)) module for map projection (UMPACK)
    * ((<Text|NumRu::AdvancedDCL::Text>)) module for drawing characters (SGPACK)
    * ((<Arrow|NumRu::AdvancedDCL::Arrow>)) module for drawing arrows (SGPACK)
    * ((<Mark|NumRu::AdvancedDCL::Mark>)) module for drawing markers (UUPACK)
    * ((<ThreeD|NumRu::AdvancedDCL::ThreeD>)) module for drawing 3D figures (SCPACK)
    * ((<Bar|NumRu::AdvancedDCL::Bar>)) module for drawing bar graph (UUPACK)
    * ((<Box|NumRu::AdvancedDCL::Box>)) module for drawing box graph (UUPACK)
    * ((<Gap|NumRu::AdvancedDCL::Gap>)) module for filling gap between two lines (UUPACK)
    * ((<Errbar|NumRu::AdvancedDCL::Errbar>)) module for drawing error bar (UUPACK)

==The way of using this library
It is assumed that this library is installed. (A way to install is written in INSTALL.)

If you use this library, you type in programs as follows.
  require 'advanceddcl'

===basics of this library
* ((<Device and Frame>)) 
* ((<Coordinate System and Transformation>)) 
* ((<2-D Normalization Transformation>)) 
  * ((<Types of Coordinate Systems>)) 
  * ((<Map Projections>)) 
  * ((<Parameters for Normalization Transformation>)) 
  * ((<Settings of the U-Coordinate System>)) 
* ((<3-D Normalization Transformation>)) 
  * ((<Types of Coordinate Systems>)) 
  * ((<Parameters for 3-D Normalization Transformation>)) 
  * ((<Setings for Coordinate Systems>)) 
* ((<Perspective Transformation>)) 
  * ((<Transformation Parameters>)) 
  * ((<Settings for Perspective Transformation>)) 
* ((<Output Primitives>)) 
  * ((<Line Indices>)) 
  * ((<Line Types>)) 
  * ((<Tone Pattern Number>)) 
  * ((<Interpolation and Clipping>)) 
  * ((<Caution on Clipping Tone Patterns>)) 
* ((<Layout>))

===Device and Frame

To draw a graphic, you will first need a device such as a display or
printer.  The process of preparing the device is called "open" a
device, and to "close" the device means to end the graphic processing.
All graphics must be drawn between this "open" and "close" process.
In the DCL, a single device can be opened at one time.  Two or more
devices cannot be opened simultaneously, but it is possible to select
one device from several while open.

The graphics drawn on the device may extend over several pages.  In
the DCL, the term "<b>frames</b>" are used instead of the pages.  This
is because the concept of pages do not exists on some devices (for
example, in devices that with outputs on long sheets like scrolls), or
there are cases where several frames are allotted to a single physical
page, as explained in a later section, and a frame needs to be
distinguished from a physical page.

Programs that use graphics follow the flow below.
   Dev.open("X")           ! Open device number "X"
   Frame.new               ! Prepare frame
   . . . . . . . .         ! Graphics program for the first page
   Frame.new               ! New page
   . . . . . . . .         ! Graphics program for second page    
      . 
      . 
      . 
   Dev.close               ! Close device

===Coordinate System and Transformation

There are 4 levels to the coordinate systems used in this library. The levels are labeled with symbols, U, V, R, and W, in descending order, and are referred to as the "U-coordinate", "V-coordinate", "R-coordinate", and "W-coordinate", respectively. (See Fig. 1.3) The letters U, V, R, and W are derived from the names of the systems below, and terms such as "U-coordinate" will be used as abbreviations. (These terms may further be shortened to "UC".)                    

* U : User coordinate
* V : Virtual rectangular coordinate
* R : Normalized rectangular coordinate
* W : Workstation coordinate

In the explanation of arguments in the reference manual, the coordinates for the U-coordinate is written as (UX,UY), with characters representing the coordinate system. (The normalized rectangular coordinate is called an R-coordinate instead of the N-coordinate based on the rules of FORTRAN concerning floating-point variables. 
   Relationship Between Coordinate System and Transformation Functions
                      2D coordinate  3D coordinate
   User coordinate          UC            UC3
                             |             |
                           (TC)            | normalization transformation
                             |             |
   Virtual rectangular      VC------------VC3
        coordinate           |             | perspective transformation
                             |             | 
   Regular rectangular      RC-------------
        coordinate           |               workstation transformation
                             |
   Device coordinate        WC

The highest-level U-coordinate includes many systems from the normal X-Y coordinate to map projection coordinates. In the V-coordinate, one level below the UC, a rectangular coordinate is defined, one for 2-D and 3-D each, and the various UC systems are transformed to these 2 types. This transformation is called "the normalization transformation".   

On the other hand, the W-coordinate, which is the lowest-level system, is a device-specific coordinate, and is different according to device. To unify these coordinate systems, the R-coordinate is defined one level above the WC. All coordinates are transformed to WC via the RC. This transformation is called the "workstation transformation". The RC is a 2-D rectangular coordinate, and the defined range for both the x- and y-axes are [0,1].        

The transformation that connects the higher two and lower two levels of coordinate systems, or one from the VC to RC, is called the "perspective transformation". The perspective transformation is, true to its name, a transformation using the perspective representation, and is basically a transformation from the 3-D VC to the 2-D RC. The 2-D VC is first allocated to a 3-D VC before perspective transformation. This 2-D VC can be allotted to an arbitrary plane that is perpendicular to the x-, y-, or the z-axis in the 3-D system. Through this, a 2-D plot may be drawn from an oblique viewpoint. 

In the initial state, the perspective transformation from the 2-D VC is an identical transformation, so no special attention is needed in perspective transformation as long as the 2-D coordinate system is treated normally. However, there may be a need to use the perspective transformation later on, so the user is advised to distinguish between the VC and the RC. In other words, "information accompanying the plot" such as axes, should be transformed along with the plot in the perspective transformation, so it should be drawn in the VC. "Information accompanying the page" such as page numbers and values of parameters drawn by SLPACK should be drawn in the RC. 

The normalization transformation of map coordinate systems are carried out in two steps: "rotation" and "projection". The intermediate coordinate system (the rotated UC) is called the T-coordinate (Terrestrial coordinate). In the TC, the defined ranges of the x-axis and y-axis are [-180, 180] and [-90, 90], respectively. All maps are transformed according to the definitions for various map projections from this coordinate system. To select an arbitrary longitude as the central longitude, or to make projections on the transverse aspect or the oblique aspect, the coordinate system must be rotated before the projection. 

Since the TC is a temporary work coordinate system during the normalization transformation, the user normally does not need be aware of it. However, since <b>clipping of the map coordinates</b> is performed in the TC and not in the UC, one needs to take the TC into consideration during clipping.

===2-D Normalization Transformation
===Types of Coordinate Systems

The following types of 2-D coordinate systems exist in the UC, and the
user can specify one with the number of function. (Functions that
return the function number from an abbreviation or name is also
available.) Each coordinate system needs different sets of parameters
for normalization transformation depending on the type.

* Rectangular Linear Coordinates 
    Number  Abbrev.  Name   
    1 U-U UNI-UNI rectangular uniform coordinate 
    2 U-L UNI-LOG semi- logarithmic coordinate 
    3 L-U LOG-UNI semi-logarithmic coordinate 
    4 L-L LOG-LOG logarithmic coordinate 

* Rectangular Curvilinear Coordinates 
    Number  Abbrev.  Name  
    5 POL POLAR polar coordinate 
    6 BPL BIPOLAR bipolar coordinate 
    7 ELP ELLIPTIC elliptic coordinate (not yet implemented) 

* Map Projection Coordinates
    Number  Abbrev.  Name     
    10 CYL CYLINDRICAL equidistant cylindrical projection 
    11 MER MERCATOR Mercator's projection 
    12 MWD MOLLWEIDE Mollweide's projection 
    13 HMR HAMMER Hammer's projection
    14 EK6 ECKERT Eckert VI projection
    15 KTD KITADA Kitada's elliptic projection 
    20 CON CONICAL equidistant conical projection 
    21 COA CONICAL EQ.AREA Lambert's equal-area conical projection 
    22 COC CONICAL CONFORMAL Lambert's conformal conical projection 
    23 BON BONNE Bonne's projection 
    30 OTG ORTHOGRAPHIC orthographic projection 
    31 PST POLAR STEREO polar stereo projection 
    32 AZM AZIMUTHAL azimuthal equidistant projection 
    33 AZA AZIMUTHAL EQ.AREA Lambert's azimuthal equal-area projection 

* User Defined Coordinates
    Number  Abbrev.  Name   
    99 USR USER User defined function

The user will prepare the transformation function necessary for the user defined coordinate system.     

===Map Projections
In the DCL, map projection coordinates are treated as normal U-coordinates. There are many different types of map projections, and a whole book would be needed to explain all of them. Here, we will introduce the fundamentals that you should know for using the DCL library. For details on the map projections, the user is referred to "Map Projections" by Masahichi Nomura. (This book is out of press.) 

A map projection is basically a method to "project a spherical surface onto a 2-D plane." Since the Earth is actually nearer to a spheroid and not a perfect sphere, a spheroid must be projected onto a plane in the strict sense of map projection. However, this library supports projection from a spherical surface, and not from a spheroid. The projected object may have 3-D characteristics depending on the projection method used, but keep in mind that the map projection coordinates are only 2-D.  

In many map projection methods, geometric techniques are used to project the spherical surface onto a plane, and the methods can be classified into several groups according to the morphology of the plane. 
* Cylindrical projection
  the globe is positioned inside a cylinder with the same radius,and the latitudes and longitudes are projected onto the surrounding cylinder, which is then spread out.
* Conic projection
  the globe is positioned in a right circular cone, and the latitudes and longitudes are projected onto the cone, which is then spread out.     
* Azimuthal projection
  the globe is placed on a plane touching the globe at one point,and the latitudes and longitudes are projected onto this plane.   
* Conventional projection
  Methods other than the above

When the axes of these cylinders and cones, or the perpendiculars from the contact in azimuthal projections coincide with the Earth's axis of rotation, the projection is on normal aspect when they are perpendicular to the axis of rotation, the projection is on transverse aspect, and when they are at oblique angles, the projection is on oblique aspect Generally, cylindrical projections are suited for displaying the whole Earth, and conic projections for displaying the mid latitudes. 

Since spherical surfaces are projected onto a plane in a map projection, distortions always exist to some extent. Therefore, many projections are designed to preserve either the area or angle. In principle, it is impossible to preserve both area and angle in map projections. The projections are classified into the following groups according to the properties they preserve

* equal-area projection
the relative area is displayed properly in all parts of the map
* conformal projection
the local angle is displayed properly in all parts of the map
* equidistant projection
the distance is displayed properly along the latitudes, longitudes, and azimuth lines.

The following map projections are supported in this library. 
* Cylindrical Projections
    Number       Projection         Equal-area   Conformal   Equidistant
     10   equidistant cylindrical       n            n         meridian
                projection  
     11   Mercator's projection         n            y            n
     12   Mollweide's projection        y            n            n
     13   Hammer's projection           y            n            n
     14   Eckert VI projection          y            n            n
     15   Kitada's elliptic projection  y            n            n
* Conic Projections
    Number       Projection         Equal-area   Conformal   Equidistant
     20   equidistant conical           n            n         meridian
               projection
     21   Lambert's equal-area          y            n            n 
           conical projection
     22   Lambert's conformal           n            y            n
          conical projection
     23   Bonne's projection            y            n            n
* Azimuthal Projection
    Number       Projection         Equal-area   Conformal   Equidistant
     30   orthographic projection       n            n            n
     31   polar stereo projection       n            y            n
     32   azimuthal equidistant         n            n      azimuthal line
                projection
     33   Lambert's azimuthal           y            n            n
          equal - area projection

In cartography, Hammer's projection is classified as a variant of the Lambert's azimuthal equal - area projection, but its form and use are more similar to cylindrical projections, so it will be classified under cylindrical projections here, for convenience. All of these projections can be shown on normal, transverse, and oblique aspects.   

The satellite view, which is a variant of an orthogonal projection and resembles the Earth as seen from a satellite, is also supported in the library, although it is usually not considered as a normal map projection. A normal orthogonal projection views the Earth from a line at infinity, but the satellite view views the Earth from a finite point.
 
===Parameters for Normalization Transformation
The following are the parameters necessary for normalization transformation of U-coordinate into 2-D V-coordinate . Each parameter can be changed/inquired by Fig module, although utilities are available for setting similar parameters at once. 

:Viewport (VXMIN, VXMAX, VYMIN, VYMAX)
   This set of parameters that specifies the viewport is common for all transformations and is requisite. A viewport is usually a rectangular frame in which the axes are drawn. When clipping is specified, graphics outside the clipping rectangle is not drawn. The coordinates in VC for the lower left-hand and upper right-hand corners are used to specify the viewport. 

:Window (UXMIN, UXMAX, UYMIN, UYMAX)
   This set of parameters is necessary for setting the rectangular linear coordinate. This is specified using the coordinates in UC corresponding to the viewport.         

In map projection coordinates, this information is not needed for GRPH1, but the GRPH2 package may inquire about this as "the lat-long range in scope." In this case, the viewport and the window does not usually coincide. These parameters can be changed/inquired with parameter "window" in Fig module.

:Similar Transformation Parameters (SIMFAC, VXOFF, VYOFF)
   A set of parameters for similar transformation (scaling and origin shift) in a rectangular curvilinear coordinate and map projection transformations, defined by the function MATH1/CTRLIB for rectangular curvilinear coordinates, and MATH1/MAPLIB for map projection coordinates. In GRPH1, the value returned by these functions is multiplied by <tt>SIMFAC</tt> to convert to a value in VC, and is set at the position with the origin shifted parallel from the center of the viewport by (VXOFF, VYOFF). These parameters can be set/inquired with 'simtrans' method in Fig module. 

: Map Coordinate Rotation Parameters(PLX, PLY, PLROT)
   Map projection transformations are always carried out within the longitudinal range of [-180, 180] and latitudinal range of [-90, 90], so to project from an arbitrary viewpoint, the lat-long coordinates must be rotated before the projection. <tt>(PLX, PLY)</tt> specifies the latitude and longitude (UC) for positioning the pole of the projection coordinate (TC), and <tt>PLROT</tt> specifies the angle of rotation of the projection coordinates around the pole.                    

   The relationship between these parameters and the "Euler's angle<tt>( theta , phi , psi )</tt>" used generally to specify the angle of rotation in 3-D is     
   <tt> theta = pi /2-PLY,  phi =PLX,  psi =PLROT</tt>. (Consult math dictionaries for a definition of the Euler's angle.)                       

   These parameters can be changed/inquired with parameter "mapangle" in Fig module.
 
   The actual procedures are as follows. In the initial state, UC coincides with TC (projection coordinate).          
   (1)TC is rotated around the north pole by  phi . (The meridian at <tt>PLX</tt> longitude is set as the central meridian.)                       
   (2)The north pole of TC is rotated in the direction of the central meridian by <tt> theta </tt>. (The point at long. and lat.<tt>(PLX, PLY)</tt> is considered as the north pole of TC.)                       
   (3)The projection is rotated around the north pole of the TC by  psi . (The angle between the meridian at longitude <tt>PLX</tt> and the central meridian of TC is <tt>PLROT</tt>.)            

   The transformation from TC to VC is determined by the similar transformation parameters above. The value of TC projected onto the origin of VC depends on the type of map projection.

           Projection          TC projected onto origin
   * Cylindrical Projection              (0, 0)  
   * Azimuthal Projection                (90, 0)
   * Conic Projection                  Apex of Cone

   The value of TC projected onto the origin corresponds to the case when the Euler's angle is <tt>(0,0,0)</tt>, or when (<tt>PLX,PLY,PLROT)=( lambda ,90.,0)</tt>.          

   To set the meridian at <tt> lambda </tt>  as the central meridian in a normal cylindrical or a conic projection (on normal aspect), let (<tt>PLX,PLY)=( lambda ,90.,0)</tt>.                  

   To draw a map with long. and lat. of <tt>( lambda , phi )</tt> at the center in an azimuthal projection, let (<tt>PLX,PLY)=( lambda , phi )</tt>, and specify <tt>PLROT</tt> as required. When <tt>PLROT=0</tt>, the south pole is always below the origin.                     

   In a cylindrical projection on the traverse aspect, specify a point on the Equator as <tt>(PLX,PLY)</tt>. When <tt>PLROT=0</tt>&nbsp;, the south pole of UC is projected onto the central meridian. For example, if you wish to present the Arctic Ocean, the Atlantic Ocean, and the Southern Ocean as a continuous body with the traverse aspect, let <tt>(PLX, PLY, PLROT) = (60.,0.,-90.)</tt>.                     
:Standard Latitude (STLAT1, STLAT2)
   In the conic projection, the standard latitude is necessary besides the above parameters. Since the Lambert's conformal conical projection has two standard parallels, <tt>STLAT2</tt> also needs to be specified. In all other conic projections, only <tt>STLAT1 </tt> is used.  

These parameters can be inquired/changed with Fig module.  
 
:Radius of Satellite Orbit(RSAT)
   Originally, the orthographic projection views the Earth from line of infinity. But a projection that view the Earth from a satellite at a finite point is also available as option (Satellite View). To select this option in the orthographic projection, specify <tt>RSAT</tt> to a value larger than 1. <tt>RSAT</tt> is the "radius of the satellite orbit" when 1 is the radius of the Earth.      
This parameter can be inquired/changed with Fig module.  

===Settings of the U-Coordinate System
Before drawing graphics in a U-coordinate, the <tt>Frame.new</tt> must be called and the parameters that determine the transformation functions must also be specified. The parameters that must be specified depends on the coordinate system, but the main parameters are all handled by Fig module.          

To set the parameters for a transformation function, the following coding is performed after <tt>Frame.new</tt>. 
* Rectangular Coordinate       
   Fig.viewport=[VXMIN, VXMAX, VYMIN, VYMAX]
   Fig.window=[UXMIN, UXMAX, UYMIN, UYMAX]
   Fig.transnum=ITR

* Orthogonal Coordinate     
   Fig.viewport=[VXMIN, VXMAX, VYMIN, VYMAX]
   Fig.simtrans=[SIMFAC, XOFF, YOFF]
   Fig.transnum=ITR

* Map Projection Coordinate  
   Fig.viewport=[VXMIN, VXMAX, VYMIN, VYMAX]
   Fig.window=[SIMFAC, XOFF, YOFF]
   Fig.mapangle=[PLX, PLY, PLROT]
   Fig.transnum=ITR

In conic projections, the latitude <tt>(STLAT1, STLAT2)</tt> of the standard parallel must be specified with <tt>SGRSET</tt> besides the above.   

These parameters can be specified in any order. A Function is also available which returns the number of transformation function when inquired using the function name or abbreviation.        

===3-D Normalization Transformation
===Types of Coordinate Systems
The following types of coordinates exist as user-defined coordinate systems (UC), and the user selects one by specifying the function number. 

Number
(1) Rectangular Coordinate
(2) Cylindrical Coordinate
(3) Spherical Coordinate

In contrast to 2-D coordinate systems, logarithmic axes are specified in 3-D systems with the log switch (see later section) and not by the function number.  

===Parameters for 3-D Normalization Transformation
The following parameters are necessary for normalization transformation of  
the 3-D U-coordinate to the 3-D V-coordinate. These can be set/inquired separately using Fig module.

:Viewport (VXMIN3, VXMAX3, VYMIN3, VYMAX3, VZMIN3, VZMAX3)  
   A set of parameters necessary for transforming rectangular linear coordinate (<tt>ITR=1</tt>), and is set together with the window parameters. Unlike the viewport in 2-D, it has no connection to the clipping range. These parameters can be changed/inquired with parameter "viewport3d" in Fig module.  

:Window (UXMIN3, UXMAX3, UYMIN3, UYMAX3, UZMIN3, UZMAX3) 
   When <tt>ITR=1</tt>, the coordinate values corresponding to the viewport is specified with the values in UC. These parameters can be changed/inquired with parameter "window3d" in Fig module.  

:Log Switch (LXLOG3, LYLOG3, LZLOG3)
  When <tt>ITR=1</tt>, set <tt>.TRUE.</tt> to make both axes presented in log. 
These parameters can be changed/inquired with parameter "log3d" in Fig module. 

:Similar Transformation Parameters (SIMFAC3, VXORG3, VYORG3, VZORG3)
  A set of parameters for similar transformation (scaling/origin shift) specified in the case of cylindrical and spherical coordinates. The origin is set as (VXORG3, VYORG3, VZORG3). These parameters can be changed/inquired with parameter "origin" in Fig module.

===Setings for Coordinate Systems
To draw graphics in a 3-D UC, as in the 2-D coordinate system, the parameters that determine transformation function must be specified after calling <tt>Frame.new</tt> but before starting to draw graphics. The parameters for 3-D systems are independent of those for the 2-D system.     

The main parameters are all handled by Fig module.        

but since it is cumbersome to set the parameters for transformation function 
individually, a routine is available for setting the parameters at once, which is usually used. To set transformation parameters, the following coding is performed after <tt>Frame.new</tt>.    
* Rectangular Coordinate     
   Fig.viewport3d=[VXMIN3, VXMAX3, VYMIN3, VYMAX3, VZMIN3, VZMAX3]
   Fig.window3d=[UXMIN3, UXMAX3, UYMIN3, UYMAX3, UZMIN3, UZMAX3]
   Fig.transnum3d=ITR3

* Cylindrical Coordinate, Spherical Coordinate       
   Fig.origin=[SFAC3, VXORG3, VYORG3, VZORG3]
   Fig.transnum3d=ITR3

===Perspective Transformation
Perspective transformation produces perspective by viewing the 3-D space from 
an arbitrary eye point. In the DCL library, it refers to the transformation from a 3-D VC to 2-D RC.

Generally, parallel lines in 3-D space projected onto a 2-D plane are not   
parallel. If the eye point is placed at the line of infinity, parallel lines   
will be projected as parallel lines, but graphics projected in such a manner   
seem unnatural.  

To perform perspective transformation, the&nbsp; <b>"eye point"</b> and the <b>"center of focus"</b> must be set. The eye point is the position from which the 3-D space is viewed, and can be thought of as the position of the camera lens. The center of focus, on the other hand, is not the focus on the film inside the camera, but is the point being viewed from the eye point. The line connecting the eye point and the center of focus is the <b>"line of sight"</b>  .      

With the perspective transformation, a point in a 3-D graphic is projected  
onto a point on a plane where the line connecting the point in 3-D space and the eye point intersects with the <b>"plane of projection"</b>. In the DCL, the 
plane of projection is the plane passing through the center of focus and 
perpendicular to the line of sight.    

In the case of 2-D coordinate systems, perspective transformation can be made 
by allocating the 2-D VC to a plane in a 3-D VC. The plane onto which the 2-D 
plane can be allocated is one which is perpendicular to either the X-, Y-, or 
the Z-axis, and cannot be allocated to an oblique plane. (To view the plane from an oblique angle, move the eye point.)

===Transformation Parameters
:Eye point (XEYE3, YEYE3, ZEYE3)
   An eye point in 3-D space. Think of holding a camera in this position.   

:Center of Focus (XOBJ3, YOBJ3, ZOBJ3)
   The point at which the line of sight is aimed in 3-D space. Think of it as the position of the subject of the photograph. The plane perpendicular to the line of sight (line connecting the eye point and center of focus) and passing through the center of focus is the plane of projection. In other words, point (X, Y, Z) in 3-D space is projected onto a point where the line connecting the point in 3-D space and the eye point intersects with the plane of projection.   
	
:Angle of View (ANGLE3)
   The length of the line projected onto the plane at this angle from the eye point is the unit length in a normalized rectangular coordinate system.   

:Tilt (TILT3)
   Inclination of the Z-axis. Equivalent to the tilt around the optical axis for camera. When this angle is 0, the Z-axis in 3-D space becomes parallel to the Y-axis in s normalized rectangular coordinate system. When the line of sight is parallel to the Z-axis, the Y-axis becomes the standard line. 

:Offset (XOFF3, YOFF3)
   The position of the center of focus projected onto a normalized rectangular coordinate system is specified with its location relative to center. Normally, this value is (0,0). 
     
:Position in the 2-D Plane (IXC3, IYC3, SEC3)
   Specifies where the 2-D plane will be positioned in 3-D space. <tt>IXC3, IYC3</tt> specifies the 3-D coordinate corresponding to the X and Y coordinates in the 2-D plane in the form of numbers (1, 2, 3), which represent the X, Y, and Z axes, respectively. When negative values are specified for these values, the values are assigned with inversed signs. <tt>SEC3</tt> is the remaining coordinate value.     

===Settings for Perspective Transformation
In perspective transformation, the parameters that determine transformation function are also specified after calling <tt>Frame.new</tt> and before drawing graphics. The parameters for perspective transformation are independent from those of the 2-D and 3-D normalization transformations.      

The main parameters are all handled by Fig module. To set the parameters for transformation function,  the following coding is performed after <tt>Frame.new</tt>.
   Fig.eyepoint=[XEYE3, YEYE3, ZEYE3]
   Fig.objpoint=[XOBJ3, YOBJ3, ZOBJ3]
All other parameters are specified with parameters in Fig module as necessary.        
To allocate a 2-D plane to a 3-D plane,  <pre>
   Fig.plane=[IXC3, IYC3, SEC3]
will be called. This will permit the graphics routines on the 2-D planes to be used on a plane in a 3-D space.   

===Output Primitives
The basic elements constituting a graphic are called output primitives. AdvancedDCL has the following output primitives.   
* Polyline  
* Polymarker  
* Text 
* Tone  
There are other secondary primitives besides these.       
* Line
* Arrow  
Five of these six primitives (excluding the tone primitive) all consist of line segments. Line segments have two attributes: the line index and line type. A tone has the attribute of tone pattern index. These attributes are defined as follows.   

Coordinates handled by AdvancedDCL include curvilinear coordinates, so it is not appropriate in some cases to connect coordinates specifying line segments and boundaries of tone patterns with straight lines. Therefore, a function is available that interpolates between the coordinates by dividing the space between the coordinates into several sections. Furthermore, all primitives are subject to clipping at a certain boundary.

===Line Indices
The thickness and color of a line segment drawn by AdvancedDCL is specified with a 3-digit integer (nnm) called the line index. Even when outputting to a device that can only change either the thickness or color, two lines with different line indexes can be distinguished because the line indexes follow the rules below for thickness and color. 

In a system where both the thickness and color can be changed, the first 2 digits (nn=0- 99) indicate the color index, and the last digit (m=0- 9) represents thickness. The standard color indexes from 1 to 5 indicate    

(1)white or black (foreground)
(2)red
(3)green
(4)blue
(5):yellow

Colors specified by larger indexes are defined in the color map file. The line thickness is specified by 1(fine) - 9(thick). With an output device in which only the line thickness can be changed, nn is read as m only when m=0. With an output device in which only the color can be changed, m is read as nn only when nn=0. Therefore, except in cases where the line thickness and color needs to be specified, specification with a single-digit index will make the lines distinguishable in a device-dependent method.     
 
===Line Types
Line types indicate solid, dashed, and doted lines. The following line types are allotted to numbers 1 through 4.  
(1)solid
(2)dashed
(3)dotted
(4)alternate long and short dash

All other integers besides 0 specify a line type which uses the bit pattern of the lower N bits (N is a value determined by the internal variable <tt>'NBITS'</tt>. Initial value is 16; For example, when N=16 and <tt>ITYPE</tt> = Z'0000F0F0' (hexadecimal constant), the line type is specified as '<u>4bits ON</u> 4bits OFF <u>4bits ON</u> 4bits OFF'. The length of 1 bit is determined by the parameter <tt>bitlen</tt> in Plot module. 

===Tone Pattern Number
Tone pattern indexes contain information on both color and pattern. The last 3 digits specify the pattern index and all preceding digits specify the color. When the color is not specified (specification only by 3 digits), the foreground color (color index 1) is used. 

The first of the three digits for the pattern index represent the pattern type.
(0)Dot
(1)Horizontal hatch
(2)Right-up hatch
(3)Vertical hatch
(4)Left-up hatch
(5)Cross hatch
(6)Diagonal cross hatch

The second of the three numbers for pattern index specifies the dot size or line thickness with numbers from 0 to 5. Larger values indicate larger dots or thicker lines. The last digit specifies the density of the dots or diagonal lines with numbers from 0 to 5. Larger values indicate higher densities of dots or lines. No dots or lines are drawn for 0. 

Pattern index 999 specifies a solid fill. 

With color displays and printers, solid fill patterns with specified colors are often used. There is an option that will convert the color indexes into different pattern indexes for dot patterns so that the fill areas can be distinguished even when such programs are executed on environments that do not support color.   

There are no patterns defined for integer values other than for the combinations above.    

===Interpolation and Clipping
There are 2 interpolation functions. 

* Linear Interpolation:
   Linear interpolation is performed in the user-coordinate system. It can be specified in all coordinate systems. However, specification in a normal rectangular-uniform coordinate (<tt>ITR=1</tt>) is meaningless; and it is frequently used in drawing axes in curvilinear coordinates and parallels in map coordinates.
* Great Circle Interpolation:
   Interpolation in map projection coordinates (TC) using a great circle passing through the starting and end points. This is effective only with map projection functions. 

In a map projection function, the singular point is always at the pole in TC, so by specifying a great circle, the line segments in the vicinity of the singular point can be more appropriately drawn. The following are the parameters for Fig module.  
* llnint (true or false)
   A flag specifying whether linear interpolation will be made. (Default value: <tt> .FALSE.</tt>)
* LGCINT (true or false)
   A flag specifying whether a great circle interpolation will be made. (Default value: <tt>.TRUE.</tt>). 
* RDX, RDY (Float)
   The interpolation interval. The length of a segment in the interpolation will not exceed this value. (Default value: (5., 5.))

Clippings will be performed in the following locations. 
* Boundary of a map Map projection coordinates (TC)     
* Viewport Virtual rectangular coordinate (VC)      
* Boundary of a device Normalized rectangular coordinate (RC)      

The clipping is done around a clipping rectangle with sides parallel to the axes of the coordinate system. Therefore, when the pole is moved in a rotation of a map projection, the clipping boundaries in the TC will not coincide with the meridians and parallels.    
  
The following are the parameters in Fig module for this function. 

* lclip (true or false)
   A flag specifying whether clipping will be made in the viewport. (Default value: <tt>.FALSE.</tt>)
* viewport(VXMIN, VXMAX, VYMIN, VYMAX) (Float)
   The viewport. Normally set by <tt>Fig.viewport=</tt>.
* mapwindow(TXMIN, TXMAX, TYMIN, TYMAX) (Float)
   The clipping boundaries in TC. (Default values:&nbsp; -180.0, +180.0, -90.0, +90.0)
* irmod (Fixnum)
   A parameter specifying whether the tone boundary lines will be connected in the clockwise or counter-clockwise direction. (Default value:<tt> 0</tt>).

The clipping of a viewport can be switched ON/OFF with <tt>lclip</tt>. Other clippings will always be performed. In orthographic projections, the line segments on the reverse side will also be drawn when the default value for the clipping boundary in TC is used. To draw only lines on the front side, set <tt>'TYMIN'</tt> to 0.         

===Caution on Clipping Tone Patterns
The tone patterns in the DCL normally (if not clipped) fill a polygon even when the specified boundaries of the polygons overlap. More precisely, soft fill is performed following the Even Odd Rule and hard fill is machine-dependent, but even the hard fills are made so as to follow the rules of soft fills as much as possible.     

In earlier versions, clippings could not be made on a hard fill, but from DCL ver.5, <b>clippings can be made on hard fill in a way similar to a soft fill</b>. In other words, the clipping of the tone pattern is not performed by the hardware. Instead, boundary points specifying the overlap in the specified region and the clipping range are generated, and the data are handed over to the tone routine. The algorithm for calculating this clipped boundary line  runs on the precondition that the direction of the boundary line is known. Here, the direction of the boundary line is the clockwise or counter-clockwise direction in which the sequence of points defining the boundary is specified. Although this algorithm has poor compatibility with traditional rules for fill tone patterns, it was created nonetheless since the direction of the boundary line on the map (TC) must be provided to determine the outside and inside of the boundary.   

The actual procedure of tone clipping is as follows.
(1)The boundary line is clipped and divided into several line segments. 
(2)The segmented lines are reconnected to form a closed boundary. 
   When IRMODE=0</tt>, the line segments are connected in the counter-clockwise direction; when <tt>IRMODE=1</tt>, in the clockwise direction.
(3)The generated boundary line is filled in following traditional rules.
   The traditional rules are applied to regions which had not been clipped, and <tt>IRMODE</tt> holds no meaning. This means that the exterior of the boundary line cannot be filled by specifying some appropriate <tt>IRMODE </tt>.     

Therefore, it is dangerous to perform clipping when filling in a polygon with overlapping boundaries. To be on the safe side, the user is advised to
set the direction of the boundary lines and use smaller, simpler polygons when clipping.     

When you wish to fill the outside of a polygon, divide the area into 2 sections so that the polygon will definitely be be clipped, reset the clipping area (viewport), and call the tone routine twice.

===Layout
The x- and y-axes in a R coordinate both range from 0 to 1, and the frame has an aspect ratio of 1:1. In contrast, actual devices have various aspect ratios. When the aspect ratios of the frame and the device are different, the workstation transformation is normally set to inscribe the largest frame at the center of the device when <tt>Frame.new</tt> is called.     

However, in some cases, you may want to use the full area of the device, or to set some margin around the figure. In such a case, the Frame and Page module are used. They are basically used for setting the workstation transformations, and it contains functions for changing the aspect ratio of the frame, as well as placing character strings such as titles in the margin, and placing several frames in a single page. Such functions are called the layout function.        

To use these layout functions, first open the device and then make the appropriate settings before calling <tt>Frame.new</tt>. For example, 
  Dev.open
  Frame.div('S', 4, 3)   ! Divide page into 4 n 3.
  Frame.new
  . . . . . . . . 
  Frame.new
  . . . . . . . . 
  Dev.close

A page will now be set to contain 12 frames.       

==Modules of this library
A module list and methods of each module are shown in following.
* NumRu::AdvancedDCL
=end

#japanese
=begin
=AdvancedDCL
{CuRubyDCL̃bp[Ƃč쐬ARubyDCLg₷
`惉CuƂȂĂBRubyDCLƂ̈Ⴂ̂ЂƂɃp^̃Zbg@
B{Cuł̓p^`惁\bh̃L[[hƂ
ZbgłBڂ((<p^̎舵ɂ>))QƂ̂ƁB

DCLIC}jÃy[W

((<URL:http://www.gfd-dennou.org/arch/dcl/dcl-f77doc/rc1/ >))

W[\͈ȉ̒ʂłB\bhꗗ((<{Cu̍\>))QƂ̂ƁB
  * W[          (ΉDCLpbP[W)
* NumRu
  * AdvancedDCL
    * ((<Common|NumRu::AdvancedDCL::Common>)) S̃p^Aʃp^B(Ȃ)
    * ((<Dev|NumRu::AdvancedDCL::Dev>)) foCX֌WB(SWPACK)
    * ((<Page|NumRu::AdvancedDCL::Page>)) y[WB(SLPACK)
    * ((<Frame|NumRu::AdvancedDCL::Frame>)) }̘g(y[W)B(SLPACK)
    * ((<Fig|NumRu::AdvancedDCL::Fig>)) }̃fUC(EBhEAr[|[g)sB(SGPACK, GRPACK, UMPACK, SCPACK)
    * ((<Axis|NumRu::AdvancedDCL::Axis>)) ܂Ŏ`B(USPACK)
      * ((<Lin|NumRu::AdvancedDCL::Axis::Lin>)) ``B(U[XYZ]PACK)
      * ((<Log|NumRu::AdvancedDCL::Axis::Log>)) ΐ`B(ULPACK)
    * ((<Dateaxis|NumRu::AdvancedDCL::Dateaxis>)) t`B(UCPACK)
    * ((<Plot|NumRu::AdvancedDCL::Plot>)) 2vbgsB(UUPACK)
    * ((<Contour|NumRu::AdvancedDCL::Contour>)) }`B(UDPACK)
    * ((<Tone|NumRu::AdvancedDCL::Tone>)) FhAg[\sB(UEPACK)
    * ((<Vector|NumRu::AdvancedDCL::Vector>)) xNg}`B(UGPACK)
    * ((<Map|NumRu::AdvancedDCL::Map>)) n}esB(UMPACK)
    * ((<Text|NumRu::AdvancedDCL::Text>)) eLXgB(SGPACK)
    * ((<Arrow|NumRu::AdvancedDCL::Arrow>)) t`B(SGPACK)
    * ((<Mark|NumRu::AdvancedDCL::Mark>)) }[J[`B(UUPACK)
    * ((<ThreeD|NumRu::AdvancedDCL::ThreeD>)) R}`B(SCPACK)
    * ((<Bar|NumRu::AdvancedDCL::Bar>)) _Ot`B(UUPACK)
    * ((<Box|NumRu::AdvancedDCL::Box>)) Ot`B(UUPACK)
    * ((<Gap|NumRu::AdvancedDCL::Gap>)) Q{̐̊ԂFh肷B(UUPACK)
    * ((<Errbar|NumRu::AdvancedDCL::Errbar>)) G[o[`B(UUPACK)

=={Cu̎gp@
{Cu͊ɃCXg[Ă̂ƂB(CXg[@INSTALLAINSTALL.jp({)Q)

gpɂ
  require 'advanceddcl'
Ƃ΂悢B

܂A
  include NumRu::AdvancedDCL
Ƃ邱ƂɂANumRu::AdvancedDCL::Contour.xxx  NumRu::AdvancedDCL ̕ȗłAContour.xxx ȂǂŊeW[悤ɂȂB
==={Ců{\
* ((<foCXƃt[>)) 
* ((<Wnƕϊ>)) 
* ((<2Kϊ>)) 
  * ((<Wn̎>)) 
  * ((<n}e@>)) 
  * ((<Kϊp^>)) 
  * ((<UWn̐ݒ>)) 
* ((<3Kϊ>)) 
  * ((<Wn̎>)) 
  * ((<3Kϊp^>)) 
  * ((<Wn̐ݒ>)) 
* ((<ϊ>)) 
  * ((<ϊp^>)) 
  * ((<ϊ̐ݒ>)) 
* ((<o̓v~eBu>)) 
  * ((<CCfNX>)) 
  * ((<C^Cv>)) 
  * ((<g[ԍ>)) 
  * ((<ԂƃNbsO>)) 
  * ((<g[̃NbsOɊւ钍>)) 
* ((<CAEg>))
 
===foCXƃt[
}``߂ɂ, ܂fBXvCv^Ȃ foCX ̏Kv. ̏鑀uI[vvƂ. , Ōɕ`I鎞̑uN[YvƂ. ׂĂ̐}`, foCXI[vĂN[Y܂ł̊Ԃ `Ȃ΂ȂȂ. n̓d]Cuł, x1̃foCXI[v邱Ƃ . 2ȏ̃foCXI[v邱Ƃ͂łȂ, I[vɂ̃foCX̒1Ił. 

foCXɏo͂}`y[Wɂ킽邱Ƃ邪, n̓d]Cuł, y[WƂt̑Ƀt[Ƃtg. , foCXɂĂ̓y[WƂTÔȂ (Ⴆ, ̂悤Ȓɏo͂悤ȃfoCX) , q̂悤ɕI1y[W̒ɕ̃t[ݒ肷邱Ƃ̂, Iȃy[WƋʂ邽߂ł. 

OtBbNgvǑ`Ԃ͎̂悤ɂȂ.
   Dev.open("X")           ! foCX"X"I[v
   Frame.new               ! t[p
   EEEEEEEE        ! Py[Wڂ̕`vO
   Frame.new               ! y[W
   EEEEEEEE        ! Qy[Wڂ̕`vO    
      E
      E
      E
   Dev.close               !foCX̃N[Y

===Wnƕϊ

{CuŎgpWnɂ 4 ̃x. ꂼ̃xɂ͏ʃx珇 U, V, R, W ƂLt, uUWnv, uVWnv, uRWnv, uWWnvƂ悤ɌĂ΂ (} 1.3 Q).  U, V, R, W Ƃ, ꂼꎟ̂悤ȖOɗR邪, ̖Ô, Ȍ͒P uUWnvȂǂ̗pgƂɂ ( UC ȂǂƂL). 

* U [UWn User coordinate 
* V zpWn Virtual rectangular coordinate 
* R KpWn normalized Rectangular coordinate 
* W uWn Workstation coordinate 

t@X}jÄ̐ł, UWn̍Wl (UX,UY) Ƃ悤, Wn\ċLqĂ. (KpWn N WnłȂ R WnƌĂԂ̂, ϐɊւ FORTRAN ̋Kɂ.) 
  Wnƕϊ֐Ƃ̊֌W
                     2W    3W
     [UW         UC           UC3
                         |            |
                       (TC)           |      Kϊ
                         |            |
     zpW       VC-----------VC3
                         |            |      ϊ
     KpW       RC-------------
                         |                   [NXe[Vϊ
     uW           WC

łʂ UWn͕ʂX-YWn}eW܂ő̍Wn܂. ̈ʂVWnɂ2, 3̂ꂼ1̒pWn`Ă, lUn̍Wׂ͂Ă2ނ̍Wnɕϊ. ̕ϊ̂ƂuKϊv ƌĂ. 

, łʂ̍Wnł W Wn̓foCXŗL̍Wn, foCXɂĈقȂ. 𓝈ꂷ邽߂, ̈ʂ R Wn`, ׂĂ R WnoR WWnɕϊ. ̕ϊu[NXe[VϊvƌĂ. RWn2̒pWn, x, y Ƃɒ`[0,1] ł. 

̏2Ɖ2̍Wn VWn R Wnւ̕ϊ uϊv ƌĂ΂. ῗ̖̒ʂ, }@Ȃ킿ߖ@ɂϊ, {I3VWn 2RWnւ̕ϊł. 2VWn, U, 3VWnɊtĂ瓧ϊ. ̎2VWntʒu, 3W x, y ܂ zɐȔCӂ̕ʂɂƂ邱Ƃł. ɂ, 2Iɕ`1̐}΂߂璭߂悤ȕ`ʂ\ɂȂ. 

Ԃł, 2VWn̓ϊ͍PϊƂȂĂ邽, 2̍Wn𕁒ʂɎg, ϊӎKv͂Ȃ, ƂœϊgȂ邱Ƃl, VWnRWn͎gĂǂ. Ȃ킿, WȂǂ́u}ɕtv, }@Ő}ƂƂɕϊׂȂ̂ V Wnŕ`悵, y[Wp^̐lȂǂ̂悤Ɂuɕtv R Wnŕ`悷ׂł. 

Ȃ, n}Wn̐Kϊ, Ẃu]vƁuev2iKōsȂ, ̒Ԃ̍W (UWn]W)  T W (Terrestrial coordinate) Ƃ. T Wx̒`悪 [-180, 180], y̒`悪[-90, 90]̍Wn, ̍Wnen}e̒`ɏ]ĕϊ. Cӂ̌o𒆉oƂ, @Ύ@̓e邽߂ɂ͓ȇOɉ]삪KvƂȂ. 

TWn͐Kϊ̍ۂ̈ꎞIȍƍWnł̂, ӎȂ΂ȂȂƂ͏, n}W̃NbsO UWnł͂ȂTWnōsȂ̂, ̍WnӎKv. 

===2Kϊ
===Wn̎
2UWnɂ͎̂悤Ȏނ, [U[͂̒֐ԍŎw肷 (, ܂͖̂֐ԍԂ֐pӂĂ). eWn, ̎ނɂĐKϊɕKvȃp[^قȂ. 

* pWn 
    ԍ      
    1 U-U UNI-UNI plW 
    2 U-L UNI-LOG БΐW 
    3 L-U LOG-UNI БΐW 
    4 L-L LOG-LOG ΐW 

* ȐWn 
    ԍ      
    5 POL POLAR ɍW 
    6 BPL BIPOLAR oȐW 
    7 ELP ELLIPTIC ȉ~W () 

* n}eWn 
    ԍ      
    10 CYL CYLINDRICAL ~}@ 
    11 MER MERCATOR Jg[}@ 
    12 MWD MOLLWEIDE Cf}@ 
    13 HMR HAMMER n}@ 
    14 EK6 ECKERT 6 GbPg6}@ 
    15 KTD KITADA kcȉ~}@ 
    20 CON CONICAL ~}@ 
    21 COA CONICAL EQ.AREA xgω~}@ 
    22 COC CONICAL CONFORMAL xgp~}@ 
    23 BON BONNE {k}@ 
    30 OTG ORTHOGRAPHIC ː}@ 
    31 PST POLAR STEREO |[[XeI}@ 
    32 AZM AZIMUTHAL ʐ}@ 
    33 AZA AZIMUTHAL EQ.AREA xgϕʐ}@ 

* [U[`Wn 
    ԍ      
    99 USR USER [U[`֐ 

[U[`Wn̓[U[Kvȕϊ֐pӂ̂ł. 

===n}e@
n̓d]Cuł͒n}WʂUWn ƂĈ. n}e@ɂ͎ɂ܂܂Ȏނ, ׂĂ邽߂ɂ1̖{Kvł邪, ł͒n̓d]Cu𗘗p邽߂ɒmĂׂ{I . Ȃ, n}w̏ڂ 쑺   un}e@v ȂǂQlɂ ꂽ (, ̖{͐). 

n}e@͊{IɁuʂ炩̕@2ʂɓev@ . ۂ̒n͊Sȋʂł͂Ȃ]ȉ~̂ɋ߂̂, Ȓn}esȂ߂ɂ͉]ȉ~̂̕\ʂ𕽖ʂɓeȂ΂ȂȂ. , {Cuł͉]ȉ~̂̓eł͂Ȃ, ʂ̓ê݂T|[g. Ȃ, eꂽ}`͓e@ɂĂ3Iȕ͋C, n}eW͂܂ł2Wł邱Ƃɒӂꂽ. 

n}ȇ̐}@ł͊􉽊wIȕ@ŋʂ畽ʂɓe. ̍ۂ̕ʂ̌`ɂĎ̂悤ɂ̐}@ɕނ. 
* ~}@
  nVɐڂ~̒Ɉʒut, 炩̕@Ōoܐ~ɓe, WJ}@.
* ~}@
  nVɐڂ钼~̒Ɉʒut, 炩̕@Ōoܐ~ɓe, WJ}@.
* ʐ}@
  nV̈_łɐڂ镽ʂݒ肵, 炩̕@Ōoܐ̕ʂɓe}@.
* ֋X}@
  LȊO̐}@.

̉~~̎, ܂͕ʐ}@ɂړ_̐ n̎]ƈv̂𐳎@, ̂@, ΂߂Ɍ̂Ύ@ƌĂ. ʂ, ~}@͑S̕\ɌĂ, ~}@͒ܓx̕\ɌĂ. 

n}eł͋ʂ𕽖ʂɓe邽, ǂĂЂ݂Ă܂. , ̐}@ł ʐ ܂ px ̂ǂ炩 ۑ悤ɍHvĂ. I, ̓𓯎ɕۑ悤ȓe͕s\ł. ̗lȕۑɊւ鐫ɂĎ̂悤ɕނ. 

* ϐ}@
  n}̂ǂłʐϊ֌W\}@.
* p}@
  n}̂ǂłǏIȊpx\}@.
* }@
  o, ܐ܂͕ʐ, \ }@.

{Cuł͈ȉ̂悤Ȓn}eT|[g. 
* ~}@ 
    ԍ          }@            p   
     10       ~}@      ~     ~    o
     11     Jg[}@    ~          ~
     12      Cf}@          ~     ~
     13       n}@           ~     ~
     14    GbPg6}@         ~     ~
     15       kcȉ~}@           ~     ~
* ~}@ 
    ԍ          }@                 p   
     20       ~}@           ~     ~    o
     21    xgω~}@         ~     ~
     22    xgp~}@    ~          ~
     23        {k}@                 ~     ~
* ʐ}@ 
    ԍ          }@                 p   
     30         ː}@             ~     ~     ~
     31    |[[XeI}@      ~          ~
     32       ʐ}@           ~     ~   ʐ
     33   xgϕʐ}@          ~     ~

n}@͒n}w, xgϕʐ}@̕ώƂĕʐ}@ɕނ邪, ̌`pr͂ނ~}@ɋ߂̂, ł͕֋XIɉ~}@ɕނ. ̐}@ׂĂɂ, @, , Ύ@\ł. 

Ȃ, ʂ̒n}e@ɂ͊܂܂Ȃ, ː}@̕ό`ƂĐlHq璭߂悤ȓe@ (Satellite View) T|[gĂ. ʂ̐ː}@͒nV𖳌璭߂悤ȓeɂȂĂ邪, L̈ʒu璭߂悤ȓeɕό`̂ł. 

===Kϊp^
LUW2VWɕϊ KϊɕKvȃp[^ɂ͈ȉ̂̂.  Fig ɂ ݒ/QƂł.
:r[|[g ( VXMIN, VXMAX, VYMIN, VYMAX ) 
   r[|[gw肷p^, ׂĂ̕ϊɋʂŕK{ł. r[|[gƂ, ʏW``̘gł. NbsO悤Ɏw肷, ̘g͂ݏo `悳Ȃ. r[|[gw肷ɂ, ̍ƉE̊pVCɂWl w肷. ̃p[^ Figviewport ɂ ݒ/QƂł. 

:EChE ( UXMIN, UXMAX, UYMIN, UYMAX ) 
   EChE͒pWnݒ肷̂ɕKvȃp^ł. , r[|[gɑΉWl UC ̒lŎw肷. 

   n}eWn̏ꍇɂ, GRPH1 ͈̔͂ł͂̏KvƂȂ, GRPH2 ̃pbP[WuڂĂܓxox͈́vƂӖ QƂꍇ. ̏ꍇɂ, ʂɃr[|[gƃEChE͈vȂ. ̃p[^ FigWindow ɂ ݒ/QƂł. 

:ϊp^ ( SIMFAC, VXOFF, VYOFF ) 
   ȐWnђn}eϊ̏ꍇ, ϊ (gkƌ_ړ) ̃p^. ȐWn̊֐ MATH1/CTRLIB, n}eWn̊֐ MATH1/MAPLIB ̊֐ɂ `Ă. GRPH1 ł, ̊֐Ԃl, SIMFAC | VC ̒lɕϊ, _r[|[g̒S (VXOFF, VYOFF) sړʒu ݒ肷. ̃p[^ Figsimtrans ɂ ݒ/QƂł. 

:n}W]p^ ( PLX, PLY, PLROT ) 
   n}eϊ͏Ɍox͈ [-180E!-- MATH: $[-180^\circ , 180^\circ ]$ -->,180E!-- MATH: $[-180^\circ , 180^\circ ]$ -->] ܓx͈ [-90E!-- MATH: $[-180^\circ , 180^\circ ]$ -->,90E!-- MATH: $[-180^\circ , 180^\circ ]$ -->]  ɑ΂čsȂ̂, Cӂ̎_̓esȂ߂ɂ, n}esȂOɈܓxoxW]ĂKv. (PLX, PLY) ͓eW (TC) ̋ɂoxEܓx(UC) w肵, PLROT ͓eW̋ɂ̉̉]pw肷. 

   ̃p^, ʂ3̉]pw肷̂Ɏg uIC[̊p(,,)v Ƃ̊֌W, =/2-PLY, =PLX, =PLROT ł. (IC[̊p̈ӖɂĂ, wTȂǂQƂ̂.) 

   ̃p[^ Figmapangle ɂ ݒ/QƂł. 

   ۂ̑ƂĂ, UC  TC (eW) vĂԂԂƂ,       (1) kɂ𒆐SƂ TC   ]. (ox PLX ̌o𒆉oƂ)     (2) TC ̖kɂ𒆉o̕ Ɖ]. (oxEܓx (PLX, PLY) ̒n_, TC ̖kɂƂ) 
     (3) TC ̖kɂ̉,  ]. (oxPLX ̌o, TC ̒ôȂp PLROT Ƃ. ) ƂȂ. 

   TC  VC ւ̕ϊ͏L̑ϊp^ɂ ߂. ̍ےn}ẽ^Cvɂ VC _ɓe TC  lقȂ. 

      } @ _ɓe TC 
      * ~}@ (0, 0) 
      * ʐ}@ (90, 0) 
      * ~}@ ~̒_ 

   _ɓe邱 TC ̒l, IC[̊p (0,0,0)  Ƃ, Ȃ킿 (PLX,PLY,PLROT)=(,90.,0)  ꍇɑΉ. 

   ʏ̉~}@~}@ (@) , oxɂ̌o𒆉oƂɂ, (PLX,PLY)=(,90.,0)  Ƃ. 

   ʐ}@ɂ, oxEܓx (,) 𒆐SƂ n}`ɂ, (PLX,PLY)=(,)  Ƃ, Kvɉ PLROT w肷 PLROT=0 ̎, KUC̓ɂ_̉ɂȂ. 

   @̉~}@Ȃǂł, (PLX,PLY)ƂĐԓ̓_ w肷. PLROT=0  ̎, UC̓ɂoɓe. Ⴆ, kɊC, 吼m, ɊCȂ̊CƂ @ŕ\ɂ, (PLX, PLY, PLROT) = (60.,0.,-90.) Ƃ. 

:Wܓx ( STLAT1, STLAT2 ) 
   ~}@̓eł͏L̃p^ȊO, WܓxKvł. xgp~}@͕Wܐ̂, STLAT2 w肵Ȃ΂ȂȂ, ̑̉~}@ STLAT1 ̒l݂̂g. ̃p[^ Fig ɂ ݒ/QƂł. 
:qOa ( RSAT ) 
   ː}@͖{, n𒭂߂悤ȓe@ł邪, IvVƂėL̈ʒuɂq璭߂悤ȓe (Satellite View) \ł. ː}@ł̃IvVIɂ, RSAT  1 ȏ̒lɐݒ肷. RSAT , na 1 Ƃ́uq̋Oavł. ̃p[^ Fig ɂ ݒ/QƂł. 

===UWn̐ݒ
UWnŕ`悷邽߂ɂ, Frame.new Ă񂾌ł`͂߂O ϊ֐߂p[^w肵ĂȂ΂ȂȂ. w肷ׂp[^͍WnɂĈقȂ邪, vȃp^ׂ͂ Fig ɂĊǗĂ.  

̓Iɕϊp^ݒ肷ɂ Frame.new ̂ƂŎ̂悤 R[fBO.
* pWn 
     Fig.viewport=[VXMIN, VXMAX, VYMIN, VYMAX]
     Fig.window=[UXMIN, UXMAX, UYMIN, UYMAX]
     Fig.transnum=ITR

* Wn 
     Fig.viewport=[VXMIN, VXMAX, VYMIN, VYMAX]
     Fig.simtrans=[SIMFAC, XOFF, YOFF]
     Fig.transnum=ITR

* n}Wn 
     Fig.viewport=[VXMIN, VXMAX, VYMIN, VYMAX]
     Fig.simtrans=[SIMFAC, XOFF, YOFF]
     Fig.mapangle=[PLX, PLY, PLROT]
     Fig.transnum=ITR

~ȅꍇ, ȊO SGRSET ɂĕWܐ̈ܓx (STLAT1, STLAT2) ^ĂKv. 

̃p^͂ǂ̂悤ȏԂŎw肵Ă\Ȃ. ϊ֐܂͗w肵, ϊ֐ԍԂ֐pӂĂ. 

===3Kϊ
===Wn̎
3̃[U[Wn (UC) ͎̂悤Ȏނ, [U[͂̒֐ԍɂ 1 I. 

ԍ   
(1) pW 
(2) ~W 
(3) W 

3 Wn2 WnƈقȂΐ̂Ă͊֐ԍł͂Ȃ, ΐXCb` (q) ɂĎw肷. 

===3Kϊp^
L3UW3VWɕϊ鐳KϊɕKvȃp[^ɂ͈ȉ̂̂.  Fig ɂݒ/QƂł.

:r[|[g ( VXMIN3, VXMAX3, VYMIN3, VYMAX3, VZMIN3, VZMAX3 ) 
   pW (ITR=1) ̕ϊɕKvȃp^, EBh̃p^ƋɎw肷.2̃r[|[gƈقȂ NbsO͈̔͂Ƃ͊֌WȂ. ̃p[^ Figviewport3d ɂ ݒ/QƂł. 

:EChE ( UXMIN3, UXMAX3, UYMIN3, UYMAX3, UZMIN3, UZMAX3 ) 
   ITR=1 ̎, r[|[gɑΉWl UC ̒lŎw肷. ̃p[^ Figwindow3d ɂ ݒ/QƂł. 

:ΐXCb` ( LXLOG3, LYLOG3, LZLOG3 ) 
   ITR=1 ̎, ꂼ̎ΐɂƂ, true ɂ. ̃p[^ Figlog3d ɂ ݒ/QƂł. 

:ϊp^ ( SIMFAC3, VXORG3, VYORG3, VZORG3) 
   ~WƋW̎Ɏw肷 ϊ (gkƌ_ړ) ̃p^. W_(VXORG3, VYORG3, VZORG3) ɐݒ肷. ̃p[^ Figoriginɂ ݒ/QƂł. 

===Wn̐ݒ
3UWnŕ`悷邽߂ɂ, 2WnƓl Frame.new Ă񂾌ł`͂߂O ϊ֐߂p[^w肵ĂȂ΂ȂȂ. 3̃p^2̃p^Ƃ͓Ɨł. 

vȃp^ׂ͂ Fig ɂĊǗĂ. 

̓Iɕϊp^ݒ肷ɂ Frame.new ̂ƂŎ̂悤 R[fBO.
* pWn 
   Fig.viewport3d=[VXMIN3, VXMAX3, VYMIN3, VYMAX3, VZMIN3, VZMAX3]
   Fig.window3d=[UXMIN3, UXMAX3, UYMIN3, UYMAX3, UZMIN3, UZMAX3]
   Fig.transnum3d=ITR3

* ~W, W 
   Fig.origin=[SFAC3, VXORG3, VYORG3, VZORG3]
   Fig.transnum3d=ITR3

===ϊ
ϊƂ, Cӂ̎_3Ԃ𒭂߂ĉߊoϊł. d\CuɂĂ, 3VWn2RWnւ̕ϊw. 

ʂ, ̕ϊł3ԏ̕s͓eꂽ2ʏł͕sɂȂȂ. _𖳌ɎĂ, s͕sɓe邪, ̂悤ɓeꂽ}`͂ĕsRɊ. 

ϊsȂɂ, u_v Ɓuœ_Svݒ肵Ȃ΂ȂȂ. _Ƃ͓ǂŎ̒ʂ3Ԃ𒭂߂ʒuł, J̃ŸʒułƎvΗǂ. ɑ΂, œ_S̓J̃tB̏œ_ł͂Ȃ, ڂĂ_, Ȃ킿_u߂_vł. ̎_Əœ_SԐuv ł. 
ϊɂ 3}`̊e_, ̓_Ǝ_Ԑ ueʁvƌ_ɓe. n̓d]Cuł, eʂ͏œ_Sʂ, ɐȖʂł. 

2Wn̏ꍇ, 2VWn3VWn1ʂɊt邱Ƃ, ϊ\ɂȂ. ̍ۂɊt邱Ƃł镽ʂ, 3 X, Y, Z Ŵꂩɐȕʂł, ΂߂̖ʂɊt邱Ƃ͂łȂ. (΂߂Ɍɂ͎_𓮂Ηǂ) 

===ϊp^
:_ ( XEYE3, YEYE3, ZEYE3 ) 
   3ԏł̎_. ̈ʒuɃJ\ĂƎvΗǂ. 
:œ_S ( XOBJ3, YOBJ3, ZOBJ3 ) 
   3ԓŎڕW_. Jʑ̂̈ʒuƎvΗǂ. Ȃ, œ_Sʂ, (_Əœ_SԐ)ɐȖʂ eʂƂȂ. Ȃ킿, 3ԏ̓_(X,Y,Z) , ̓_Ǝ_ʂ eʂƌ_ɓe. 
:p ( ANGLE3 ) 
   _炱̊pxŌ铊eʏ̒, KpWn̒PʒƂȂ. 
:X ( TILT3 ) 
   Z ̌X. J̌̌Xɑ. ̊px0 ̎, 3ԂZ͐KpWYɕsɂ . Z ƕsȏꍇɂ, Y ɂȂ. 
:ItZbg ( XOFF3, YOFF3 ) 
   KpWnɓeꂽœ_S̈ʒu, S̈ʒuŎw肷. ʏ킱̒l (0,0) ŗǂ. 
:2ʈʒu ( IXC3, IYC3, SEC3 ) 
   2ʂ3Ԃ̂ǂɈʒut邩w肷. IXC3, IYC3 ͂ꂼ2ʂXW, YWɑΉ 3W (1,2,3) ̐Ŏw肷̂, ͂ꂼ XW, YW, ZWɑΉ. ̐lɕ̒lw肷, tɊ蓖Ă. SEC3 ͎c̍W̍Wlł. 

===ϊ̐ݒ
ϊ̃p^ Frame.new Ă񂾌ł`͂߂O ϊ֐߂p[^w肵Ă. ϊ̃p^, 2您3KϊƓƗł. 

vȃp^ׂ͂ Fig ɂĊǗĂ.̓Iɕϊp^ݒ肷ɂ Frame.new ̂ƂŎ̂悤 R[fBO.
   Fig.eyepoint=[XEYE3, YEYE3, ZEYE3]
   Fig.objpoint=[XOBJ3, YOBJ3, ZOBJ3]
̂ق̃p^͕Kvɉ SGpSETŎw肷. 

Ȃ, 2ʂ3ʂɊtɂ, 
   Fig.plane=[IXC3, IYC3, SEC3] 
Ă. ɂ, 2ʏ̕`惋[` 3ԓ̕ʏŎgƂł悤ɂȂ

===o̓v~eBu
}`\{vfo̓v~eBuƂ. GRPH1 ̏o̓v~eBuɂ͎̂̂. 
* |C(polyline) : ܂. 
* |}[J[(polymarker) : }[J[. 
* eLXg(text) : . 
* g[(tone) : p`̂ʂԂ. 
̂ق, ⏕IɎ̃v~eBu. 
* C(line) : . 
* A[(arrow) : . 
6̃v~eBû, g[v~eBuȊO5ׂ͂Đō\. ɂ̓CCfNXƃC^CvĂ΂鑮. ܂, g[ɂ, g[p^[ԍƂ. ̑, ȉ̂悤ɒ`Ă. 

Wnɂ͋ȐWn܂܂̂, g[̋Ew肷W_̊Ԃ𒼐łȂƕssNꍇ. , W_̊Ԃɕ, Ԃ@\. , ׂẴv~eBu͂鋫Eł̃NbsȎΏۂƂȂ. 

===CCfNX
`̑ƐF̓CCfNXƌĂ΂ 3̐(nnm)Ŏw肳. ̑ƐF̂ǂ炩ςȂ悤 foCXɏo͂ꍇł, CCfNẌقȂ2{̐ʂł悤ɂ邽, CCfNX͎̂悤ȋKɂđƐFɑΉÂ. 

̑ƐFς悤ȃVXeł, 2(nn=0- 99)Fԍ, 1(m=0- 9)̑\. Fԍ, 15 ܂ł͕WI 

(1) ܂͍ (tHAOEh) 
(2)  
(3)  
(4) 
(5) 

ƌ߂Ă邪, 傫ȔԍɊւĂ colormap t@C̒`ɂ. ܂m: 1()  9()ƂȂĂ. ̑ς悤ȏo͑uł, m=0̂ƂɌnnmƂēǂ݂. ܂, ̐F悤ȏo͑uł, nn=0̂ƂɌmnnƂēǂ݂. Đ̑ƐF𖾎ƂȊO, 1̃CCfNXw肵Ă, Ƃ肠uɌŗLȕ@ɂĐ͎ʉ\ƂȂ. 

===C^Cv
C^CvƂ, , jȂǂ̐ł. 14܂ł̔ԍɂ͂炩߈ȉ̃^Cv߂Ă. 
(1)  
(2) j 
(3) _ 
(4) 1_ 

̑0ȊO͉̐Nrbg (N Plotnbits Ō܂l. l16; 2.4.4ߎQ) ̃rbgp^[pĐ킪ݒ肳. ƂN=16ITYPE = Z'0000F0F0' (16i萔)̂Ƃ, '4bits ON 4bits OFF 4bits ON 4bits OFF' ̂悤Ȕjݒ肳. 1rbg̒ Plotbitlen ߂. 

===g[ԍ
g[p^[ԍɂ͐F̏, p^[̏񂪗܂܂Ă, 3p^[ԍ, ʂ̌͐Fw肷. Fw肵Ȃ (3̔ԍw) ꍇ, tHAOEh (Fԍ1) Ɖ߂. 

p^[ԍ̍ŏʌ, p^[̎ނ킷. 
(0) hbg 
(1)  
(2) ΐ (Eオ) 
(3) c 
(4) ΐ (オ) 
(5) iq (c) 
(6) iq (΂) 

2߂, hbg̑傫ΐ̑05̊ԂŎw肷. l傫Ȃɂăhbg͑傫Ȃΐ͑Ȃ. ŉʌ̓hbgΐ̖x05̊ԂŎw肷. l傫Ȃɂăhbgΐ̖x. 0̂Ƃ͉`Ȃ. 

p^[ԍƂ999w肷ƂׂʂƂȂ. 

J[̒[J[̃v^[ł, Fw肵ׂhɂh킯΂΂Ȃ. ̃vOJ[T|[gȂŎsĂ, ȂȌ킯悤Ƀhbgɂh킯Ƃĕ\悤ȃIvVpӂĂ. 

̑gȊO̐l̓p^[`ĂȂ. 

===ԂƃNbsO
ԋ@\ɂ͎2. 

* `:   
  [UWnŐ`IɕԂ. ׂĂ̍WnŎw\ł邪,  ʏ̒plW (ITR=1) Ŏw肵ĂӖ͂Ȃ. ȐWn̍W, n}Ẅܓx`ȂǂɎg. 
* ~: 
  n}eW(TC), n_ƏI_ʂ~ŕԂ. ͒n}e֐̂ݗL. 

n}e֐̏ꍇɂ, TC ɂēٓ_͕KɂɂȂĂ̂, ~Ԃw肷邱Ƃɂ, ٓ_t߂ł̐K؂ɕ`悤 ɂȂ. ̋@\Ɋւ Fig ̃p^͈ȉ̒ʂ. 
* LLNINT (L) `ԂsȂǂw肷tO (ȗl .FALSE.). 
* LGCINT (L) ~ԂsȂǂw肷tO (ȗl .TRUE.). 
* RDX, RDY (R) ԊԊu. 1ZOg̒̒lzȂ悤 Ԃ(ȗl (5., 5.)). 

NbsȌ͎ꏊōsȂ. 
* n}̋E n}eW (TC) 
* r[|[g zpW (VC) 
* foCXE KpW (RC) 

ꂼ̃NbsO, ̍Wn̍WɕsȐň͂܂` ɑ΂čsȂ. , n}e̍ۂɋɂړ]sȂ, TC ɂNbsOE͂͌oܐɈvȂȂ. 

̋@\Ɋւ Fig ̃p^͈ȉ̒ʂ. 

LCLIP (L) r[|[gł̃NbsO sȂǂw肷tO (ȗl .FALSE.). 
* viewport(VXMIN, VXMAX, VYMIN, VYMAX) (R) r[|[g. ʏ Fig.viewport=ɂݒ肳. 
* mapwindow(TXMIN, TXMAX, TYMIN, TYMAX) (R) TC ɂNbsOE (ȗl -180.0, +180.0, -90.0, +90.0). 
* IRMOD (I) g[̋ENbsO, Eɐڑ邩, ɐڑ邩 w肷p^.  (ȗl 0). 

r[|[gɂNbsO Figlclip ɂ ON/OFF ł邪, ̃NbsO͏ɂȂ悤ɂȂĂ. ː}@ TC ɂNbsOEȗl̂܂܎g, ̐Ȃǂ`ƂɂȂ. \`߂ɂ'TYMIN'  0 ɂ. 

===g[̃NbsOɊւ钍
n̓d]Cũg[͒ʏ (NbsOȂ) w肳ꂽEĂĂ, ̐}`̓hԂ. mɂ, \tgtB Even Odd Rule ɂēhԂs, n[htB͋@ˑł邪, ł邾\tgtBɋ߂KœhԂlɂȂĂ. 

o[Wł̓n[htB̓NbsȎΏۂɂȂĂȂ, DCL ver.5 n[htB\tgIɃNbsO ł悤ɂȂ. Ȃ킿, g[̃NbsO̓n[hEFAŝł͂Ȃ, w肳ꂽ̈ƃNbsO͈͂dȂ镔w肷鋫E_𐶐, ̃f[^g[[`ɓn悤ɂȂĂ. ̃NbvꂽE߂ASY, Ě܂Ă邱ƂOƂĂ. , ĚƂ͋Ew肷_, vɎw肷邩, vɎw肷邩ƂƂ. ̃ASY͋̃g[̓hԂKƐ, n} (T Wn) ł͋Ě킩ȂƓƊO𔻒f邱ƂłȂ. 

̓Iɂ̓g[NbsO菇͎̂悤ɂȂ. 
(1) ENbv, ̐ɕ. 
(2) ؒfꂽڑȂ, E𐶐. 
   IRMODE=0 ̎, v; IRMODE=1 ̎, v ɐڑ. 
(3)ꂽEɑ΂ċ̋KɂēhԂs. 
   NbsOȂ̈ɊւĂ, ̋K̂܂ܓKp, IRMODE ͉̈ӖȂ. ܂, IRMODE KɎw肷邱Ƃ, E̊OhԂƂł킯ł͂Ȃ. 

, E悤Ȑ}`hԂ, NbsOŝ͊댯ł. NbsOs, Ŝ߂ Ě߂, ȒPȐ}`ɂlɂĂ. 

Ȃ, }`̊OhԂɂ, ̐}`mɃNbsO悤ɗ̈2, NbsÖ (r[|[g) ݒ肵Ȃ, 2xg[[`Ă. 

===CAEg
RWnx, yƂ0  1̒lƂ, `͈͂ 1:1 ̏c. ɑ΂, ۂ̃foCX̏c͂낢ł. `͈͂ƃfoCX̏c䂪vȂɂ, ʏ Frame.new Ă΂ꂽiK, foCX̒ɕ`͈͂őڂ悤 [NXe[Vϊݒ肳. 

, foCX̕`̈ςɎg, t, }̉ɓKȃ}[WƂ肽ꍇ. ̂悤Ȏɂ, Frame g. Frame ͊{IɃ[NXe[Vϊݒ肷̂, `͈͂̏cςȊO, }[WɃ^CgȂǂ̕ 1̎ɕ̃t[ׂ肷@\. ̗lȋ@\CAEg@\ƌĂ. 

̃CAEg@\gɂ, foCXI[v Frame.new ĂԑO, K؂ȐݒsȂ. Ⴆ,
  Dev.open
  Frame.div('S', 4, 3)   ! 1y[WS~Rɕ
  Frame.new
  EEEEEEEE
  Frame.new
  EEEEEEEE
  Dev.close

Ƃ邱Ƃ, 1y[W12̃t[ݒ肷邱Ƃł


=={Cu̍\
ɖ{Cu̍\yъeW[̃\bhB
* NumRu::AdvancedDCL
=end
